@@ -34,6 +34,8 @@
 
 using std::string;
 
+namespace Stockfish {
+
 namespace Zobrist {
 
   Key psq[PIECE_NB][SQUARE_NB];
@@ -92,14 +94,14 @@ std::ostream& operator<<(std::ostream& os, const Position& pos) {
      << std::setfill(' ') << std::dec << "\nCheckers: ";
 
   for (Bitboard b = pos.checkers(); b; )
-      os << UCI::square(pos, pop_lsb(&b)) << " ";
+      os << UCI::square(pos, pop_lsb(b)) << " ";
 
   if (    int(Tablebases::MaxCardinality) >= popcount(pos.pieces())
       && Options["UCI_Variant"] == "chess"
       && !pos.can_castle(ANY_CASTLING))
   {
       StateInfo st;
-      ASSERT_ALIGNED(&st, Eval::NNUE::kCacheLineSize);
+      ASSERT_ALIGNED(&st, Eval::NNUE::CacheLineSize);
 
       Position p;
       p.set(pos.variant(), pos.fen(), pos.is_chess960(), &st, pos.this_thread());
@@ -374,7 +376,7 @@ Position& Position::set(const Variant* v, const string& fenStr, bool isChess960,
               {
                   Bitboard castling_rooks = gates(c) & pieces(castling_rook_piece());
                   while (castling_rooks)
-                      set_castling_right(c, pop_lsb(&castling_rooks));
+                      set_castling_right(c, pop_lsb(castling_rooks));
               }
 
       // counting limit
@@ -486,8 +488,6 @@ Position& Position::set(const Variant* v, const string& fenStr, bool isChess960,
   tsumeMode = Options["TsumeMode"];
   thisThread = th;
   set_state(st);
-  st->accumulator.state[WHITE] = Eval::NNUE::INIT;
-  st->accumulator.state[BLACK] = Eval::NNUE::INIT;
 
   assert(pos_is_ok());
 
@@ -512,7 +512,7 @@ void Position::set_castling_right(Color c, Square rfrom) {
   Square kto = make_square(cr & KING_SIDE ? castling_kingside_file() : castling_queenside_file(), castling_rank(c));
   Square rto = kto + (cr & KING_SIDE ? WEST : EAST);
 
-  castlingPath[cr] =   (between_bb(rfrom, rto) | between_bb(kfrom, kto) | rto | kto)
+  castlingPath[cr] =   (between_bb(rfrom, rto) | between_bb(kfrom, kto))
                     & ~(kfrom | rfrom);
 }
 
@@ -569,7 +569,7 @@ void Position::set_state(StateInfo* si) const {
 
   for (Bitboard b = pieces(); b; )
   {
-      Square s = pop_lsb(&b);
+      Square s = pop_lsb(b);
       Piece pc = piece_on(s);
       si->key ^= Zobrist::psq[pc][s];
 
@@ -633,7 +633,7 @@ Position& Position::set(const string& code, Color c, StateInfo* si) {
 /// Position::fen() returns a FEN representation of the position. In case of
 /// Chess960 the Shredder-FEN notation is used. This is mainly a debugging function.
 
-const string Position::fen(bool sfen, bool showPromoted, int countStarted, std::string holdings) const {
+string Position::fen(bool sfen, bool showPromoted, int countStarted, std::string holdings) const {
 
   int emptyCnt;
   std::ostringstream ss;
@@ -792,7 +792,7 @@ Bitboard Position::slider_blockers(Bitboard sliders, Square s, Bitboard& pinners
                   Bitboard asymmetricals = PseudoAttacks[~c][pt][s] & pieces(c, pt);
                   while (asymmetricals)
                   {
-                      Square s2 = pop_lsb(&asymmetricals);
+                      Square s2 = pop_lsb(asymmetricals);
                       if (!(attacks_from(c, pt, s2) & s))
                           snipers |= s2;
                   }
@@ -805,7 +805,7 @@ Bitboard Position::slider_blockers(Bitboard sliders, Square s, Bitboard& pinners
 
   while (snipers)
   {
-    Square sniperSq = pop_lsb(&snipers);
+    Square sniperSq = pop_lsb(snipers);
     Bitboard b = between_bb(s, sniperSq, type_of(piece_on(sniperSq))) & occupancy;
 
     if (b && (!more_than_one(b) || ((AttackRiderTypes[type_of(piece_on(sniperSq))] & HOPPING_RIDERS) && popcount(b) == 2)))
@@ -863,7 +863,7 @@ Bitboard Position::attackers_to(Square s, Bitboard occupied, Color c, Bitboard j
               Bitboard asymmetricals = PseudoAttacks[~c][move_pt][s] & pieces(c, pt);
               while (asymmetricals)
               {
-                  Square s2 = pop_lsb(&asymmetricals);
+                  Square s2 = pop_lsb(asymmetricals);
                   if (attacks_bb(c, move_pt, s2, occupied) & s)
                       b |= s2;
               }
@@ -1016,7 +1016,7 @@ bool Position::legal(Move m) const {
       if (!(pseudoRoyalsTheirs & ~occupied))
           while (pseudoRoyals)
           {
-              Square sr = pop_lsb(&pseudoRoyals);
+              Square sr = pop_lsb(pseudoRoyals);
               // Touching pseudo-royal pieces are immune
               if (  !(blast_on_capture() && (pseudoRoyalsTheirs & attacks_bb<KING>(sr)))
                   && (attackers_to(sr, occupied, ~us) & (occupied & ~square_bb(kto))))
@@ -1024,17 +1024,21 @@ bool Position::legal(Move m) const {
           }
   }
 
-  // st->previous->blockersForKing consider capsq as empty.
-  // If pinned, it has to move along the king ray.
-  if (type_of(m) == EN_PASSANT)
+  // En passant captures are a tricky special case. Because they are rather
+  // uncommon, we do it simply by testing whether the king is attacked after
+  // the move is made.
+  if (type_of(m) == EN_PASSANT && count<KING>(us))
   {
-      if (!count<KING>(us))
-          return true;
-
+      Square ksq = square<KING>(us);
       Square capsq = to - pawn_push(us);
       Bitboard occupied = (pieces() ^ from ^ capsq) | to;
 
-      return !(attackers_to(square<KING>(us), occupied, ~us) & occupied);
+      assert(to == ep_square());
+      assert(moved_piece(m) == make_piece(us, PAWN));
+      assert(piece_on(capsq) == make_piece(~us, PAWN));
+      assert(piece_on(to) == NO_PIECE);
+
+      return !(attackers_to(ksq, occupied, ~us) & occupied);
   }
 
   // Castling moves generation does not check if the castling path is clear of
@@ -1085,9 +1089,8 @@ bool Position::legal(Move m) const {
   if (!count<KING>(us))
       return true;
 
-  // If the moving piece is a king, check whether the destination
-  // square is attacked by the opponent. Castling moves are checked
-  // for legality during move generation.
+  // If the moving piece is a king, check whether the destination square is
+  // attacked by the opponent.
   if (type_of(moved_piece(m)) == KING)
       return !attackers_to(to, occupied, ~us);
 
@@ -1191,7 +1194,7 @@ bool Position::pseudo_legal(const Move m) const {
 
           // Our move must be a blocking evasion or a capture of the checking piece
           Square checksq = lsb(checkers());
-          if (  !((between_bb(checksq, square<KING>(us)) | checkers()) & to)
+          if (  !(between_bb(square<KING>(us), lsb(checkers())) & to)
               || ((LeaperAttacks[~us][type_of(piece_on(checksq))][checksq] & square<KING>(us)) && !(checkers() & to)))
               return false;
       }
@@ -1279,10 +1282,10 @@ bool Position::gives_check(Move m) const {
   case PIECE_DEMOTION:
       return attacks_bb(sideToMove, type_of(unpromoted_piece_on(from)), to, pieces() ^ from) & square<KING>(~sideToMove);
 
-  // The double-pushed pawn blocked a check? En Passant will remove the blocker.
-  // The only discovery check that wasn't handle is through capsq and fromsq
-  // So the King must be in the same rank as fromsq to consider this possibility.
-  // st->previous->blockersForKing consider capsq as empty.
+  // En passant capture with check? We have already handled the case
+  // of direct checks and ordinary discovered check, so the only case we
+  // need to handle is the unusual case of a discovered check through
+  // the captured pawn.
   case EN_PASSANT:
   {
       Square capsq = make_square(file_of(to), rank_of(from));
@@ -1336,8 +1339,8 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
       ++st->countingPly;
 
   // Used by NNUE
-  st->accumulator.state[WHITE] = Eval::NNUE::EMPTY;
-  st->accumulator.state[BLACK] = Eval::NNUE::EMPTY;
+  st->accumulator.computed[WHITE] = false;
+  st->accumulator.computed[BLACK] = false;
   auto& dp = st->dirtyPiece;
   dp.dirty_num = 1;
 
@@ -1426,9 +1429,12 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
               ^ Zobrist::inHand[pieceToHand][pieceCountInHand[color_of(pieceToHand)][type_of(pieceToHand)]];
 
           if (Eval::useNNUE)
+          {
               dp.handPiece[1] = pieceToHand;
+              dp.handCount[1] = pieceCountInHand[color_of(pieceToHand)][type_of(pieceToHand)];
+          }
       }
-      else
+      else if (Eval::useNNUE)
           dp.handPiece[1] = NO_PIECE;
 
       // Update material hash key and prefetch access to materialTable
@@ -1476,7 +1482,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
       {
           Bitboard b = attacks_bb(us, QUEEN, to, board_bb() & ~pieces(~us)) & ~PseudoAttacks[us][KING][to] & pieces(us);
           while(b)
-              st->flippedPieces |= between_bb(to, pop_lsb(&b));
+              st->flippedPieces |= between_bb(to, pop_lsb(b));
       }
       else
       {
@@ -1488,7 +1494,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
       Bitboard to_flip = st->flippedPieces;
       while(to_flip)
       {
-          Square s = pop_lsb(&to_flip);
+          Square s = pop_lsb(to_flip);
           Piece flipped = piece_on(s);
           Piece resulting = ~flipped;
 
@@ -1514,6 +1520,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
           // Add drop piece
           dp.piece[0] = pc;
           dp.handPiece[0] = make_piece(us, in_hand_piece_type(m));
+          dp.handCount[0] = pieceCountInHand[us][in_hand_piece_type(m)];
           dp.from[0] = SQ_NONE;
           dp.to[0] = to;
       }
@@ -1532,7 +1539,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
                                        & rank_bb(castling_rank(us))
                                        & (file_bb(FILE_A) | file_bb(max_file()));
               while (castling_rooks)
-                  set_castling_right(us, pop_lsb(&castling_rooks));
+                  set_castling_right(us, pop_lsb(castling_rooks));
           }
           else if (type_of(pc) == castling_rook_piece())
           {
@@ -1584,6 +1591,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
           {
               // Promoting pawn to SQ_NONE, promoted piece from SQ_NONE
               dp.to[0] = SQ_NONE;
+              dp.handPiece[0] = NO_PIECE;
               dp.piece[dp.dirty_num] = promotion;
               dp.handPiece[dp.dirty_num] = NO_PIECE;
               dp.from[dp.dirty_num] = SQ_NONE;
@@ -1618,6 +1626,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
       {
           // Promoting piece to SQ_NONE, promoted piece from SQ_NONE
           dp.to[0] = SQ_NONE;
+          dp.handPiece[0] = NO_PIECE;
           dp.piece[dp.dirty_num] = promotion;
           dp.handPiece[dp.dirty_num] = NO_PIECE;
           dp.from[dp.dirty_num] = SQ_NONE;
@@ -1644,6 +1653,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
       {
           // Demoting piece to SQ_NONE, demoted piece from SQ_NONE
           dp.to[0] = SQ_NONE;
+          dp.handPiece[0] = NO_PIECE;
           dp.piece[dp.dirty_num] = demotion;
           dp.handPiece[dp.dirty_num] = NO_PIECE;
           dp.from[dp.dirty_num] = SQ_NONE;
@@ -1669,19 +1679,20 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
       Square gate = gating_square(m);
       Piece gating_piece = make_piece(us, gating_type(m));
 
-      put_piece(gating_piece, gate);
-      remove_from_hand(gating_piece);
-
       if (Eval::useNNUE)
       {
           // Add gating piece
           dp.piece[dp.dirty_num] = gating_piece;
           dp.handPiece[dp.dirty_num] = gating_piece;
+          dp.handCount[dp.dirty_num] = pieceCountInHand[us][gating_type(m)];
           dp.from[dp.dirty_num] = SQ_NONE;
           dp.to[dp.dirty_num] = gate;
           dp.dirty_num++;
       }
 
+      put_piece(gating_piece, gate);
+      remove_from_hand(gating_piece);
+
       st->gatesBB[us] ^= gate;
       k ^= Zobrist::psq[gating_piece][gate];
       st->materialKey ^= Zobrist::psq[gating_piece][pieceCount[gating_piece]];
@@ -1709,7 +1720,7 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
       Bitboard blast = (attacks_bb<KING>(to) & (pieces() ^ pieces(PAWN))) | to;
       while (blast)
       {
-          Square bsq = pop_lsb(&blast);
+          Square bsq = pop_lsb(blast);
           Piece bpc = piece_on(bsq);
           Color bc = color_of(bpc);
           if (type_of(bpc) != PAWN)
@@ -1747,7 +1758,10 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
                   ^ Zobrist::inHand[pieceToHand][pieceCountInHand[color_of(pieceToHand)][type_of(pieceToHand)]];
 
               if (Eval::useNNUE)
+              {
                   dp.handPiece[dp.dirty_num - 1] = pieceToHand;
+                  dp.handCount[dp.dirty_num - 1] = pieceCountInHand[color_of(pieceToHand)][type_of(pieceToHand)];
+              }
           }
 
           // Update material hash key
@@ -1759,9 +1773,9 @@ void Position::do_move(Move m, StateInfo& newSt, bool givesCheck) {
           // Update castling rights if needed
           if (st->castlingRights && castlingRightsMask[bsq])
           {
-              int cr = castlingRightsMask[bsq];
-              k ^= Zobrist::castling[st->castlingRights & cr];
-              st->castlingRights &= ~cr;
+             k ^= Zobrist::castling[st->castlingRights];
+             st->castlingRights &= ~castlingRightsMask[bsq];
+             k ^= Zobrist::castling[st->castlingRights];
           }
       }
   }
@@ -1832,7 +1846,7 @@ void Position::undo_move(Move m) {
       Bitboard blast = attacks_bb<KING>(to) | to;
       while (blast)
       {
-          Square bsq = pop_lsb(&blast);
+          Square bsq = pop_lsb(blast);
           Piece unpromotedBpc = st->unpromotedBycatch[bsq];
           Piece bpc = st->demotedBycatch & bsq ? make_piece(color_of(unpromotedBpc), promoted_piece_type(type_of(unpromotedBpc)))
                                                : unpromotedBpc;
@@ -1927,7 +1941,7 @@ void Position::undo_move(Move m) {
       Bitboard to_flip = st->flippedPieces;
       while(to_flip)
       {
-          Square s = pop_lsb(&to_flip);
+          Square s = pop_lsb(to_flip);
           Piece resulting = ~piece_on(s);
           remove_piece(s);
           put_piece(resulting, s);
@@ -1976,7 +1990,7 @@ void Position::do_castling(Color us, Square from, Square& to, Square& rfrom, Squ
 }
 
 
-/// Position::do(undo)_null_move() is used to do(undo) a "null move": it flips
+/// Position::do_null_move() is used to do a "null move": it flips
 /// the side to move without executing any move on the board.
 
 void Position::do_null_move(StateInfo& newSt) {
@@ -1991,8 +2005,8 @@ void Position::do_null_move(StateInfo& newSt) {
 
   st->dirtyPiece.dirty_num = 0;
   st->dirtyPiece.piece[0] = NO_PIECE; // Avoid checks in UpdateAccumulator()
-  st->accumulator.state[WHITE] = Eval::NNUE::EMPTY;
-  st->accumulator.state[BLACK] = Eval::NNUE::EMPTY;
+  st->accumulator.computed[WHITE] = false;
+  st->accumulator.computed[BLACK] = false;
 
   if (st->epSquare != SQ_NONE)
   {
@@ -2015,6 +2029,9 @@ void Position::do_null_move(StateInfo& newSt) {
   assert(pos_is_ok());
 }
 
+
+/// Position::undo_null_move() must be used to undo a "null move"
+
 void Position::undo_null_move() {
 
   assert(!checkers());
@@ -2076,7 +2093,7 @@ Value Position::blast_see(Move m) const {
 
       while (attackers)
       {
-          Square s = pop_lsb(&attackers);
+          Square s = pop_lsb(attackers);
           if (extinction_piece_types().find(type_of(piece_on(s))) == extinction_piece_types().end())
               minAttacker = std::min(minAttacker, blast & s ? VALUE_ZERO : CapturePieceValue[MG][piece_on(s)]);
       }
@@ -2092,7 +2109,7 @@ Value Position::blast_see(Move m) const {
   // Sum up blast piece values
   while (blast)
   {
-      Piece bpc = piece_on(pop_lsb(&blast));
+      Piece bpc = piece_on(pop_lsb(blast));
       if (extinction_piece_types().find(type_of(bpc)) != extinction_piece_types().end())
           return color_of(bpc) == us ?  extinction_value()
                         : capture(m) ? -extinction_value()
@@ -2175,8 +2192,8 @@ bool Position::see_ge(Move m, Value threshold) const {
       if (!(stmAttackers = attackers & pieces(stm)))
           break;
 
-      // Don't allow pinned pieces to attack (except the king) as long as
-      // there are pinners on their original square.
+      // Don't allow pinned pieces to attack as long as there are
+      // pinners on their original square.
       if (pinners(~stm) & occupied)
           stmAttackers &= ~blockers_for_king(stm);
 
@@ -2192,7 +2209,7 @@ bool Position::see_ge(Move m, Value threshold) const {
           if ((swap = PawnValueMg - swap) < res)
               break;
 
-          occupied ^= lsb(bb);
+          occupied ^= least_significant_square_bb(bb);
           attackers |= attacks_bb<BISHOP>(to, occupied) & pieces(BISHOP, QUEEN);
       }
 
@@ -2201,7 +2218,7 @@ bool Position::see_ge(Move m, Value threshold) const {
           if ((swap = KnightValueMg - swap) < res)
               break;
 
-          occupied ^= lsb(bb);
+          occupied ^= least_significant_square_bb(bb);
       }
 
       else if ((bb = stmAttackers & pieces(BISHOP)))
@@ -2209,7 +2226,7 @@ bool Position::see_ge(Move m, Value threshold) const {
           if ((swap = BishopValueMg - swap) < res)
               break;
 
-          occupied ^= lsb(bb);
+          occupied ^= least_significant_square_bb(bb);
           attackers |= attacks_bb<BISHOP>(to, occupied) & pieces(BISHOP, QUEEN);
       }
 
@@ -2218,7 +2235,7 @@ bool Position::see_ge(Move m, Value threshold) const {
           if ((swap = RookValueMg - swap) < res)
               break;
 
-          occupied ^= lsb(bb);
+          occupied ^= least_significant_square_bb(bb);
           attackers |= attacks_bb<ROOK>(to, occupied) & pieces(ROOK, QUEEN);
       }
 
@@ -2227,7 +2244,7 @@ bool Position::see_ge(Move m, Value threshold) const {
           if ((swap = QueenValueMg - swap) < res)
               break;
 
-          occupied ^= lsb(bb);
+          occupied ^= least_significant_square_bb(bb);
           attackers |=  (attacks_bb<BISHOP>(to, occupied) & pieces(BISHOP, QUEEN))
                       | (attacks_bb<ROOK  >(to, occupied) & pieces(ROOK  , QUEEN));
       }
@@ -2511,7 +2528,7 @@ bool Position::has_game_cycle(int ply) const {
           Square s1 = from_sq(move);
           Square s2 = to_sq(move);
 
-          if (!(between_bb(s1, s2) & pieces()))
+          if (!((between_bb(s1, s2) ^ s2) & pieces()))
           {
               if (ply > i)
                   return true;
@@ -2655,7 +2672,7 @@ bool Position::pos_is_ok() const {
               assert(0 && "pos_is_ok: Bitboards");
 
   StateInfo si = *st;
-  ASSERT_ALIGNED(&si, Eval::NNUE::kCacheLineSize);
+  ASSERT_ALIGNED(&si, Eval::NNUE::CacheLineSize);
 
   set_state(&si);
   if (std::memcmp(&si, st, sizeof(StateInfo)))
@@ -2684,3 +2701,5 @@ bool Position::pos_is_ok() const {
 
   return true;
 }
+
+} // namespace Stockfish
