@@ -25,7 +25,7 @@
 
 #include "variant.h"
 
-namespace PSQT
+namespace Stockfish::PSQT
 {
 
 extern Score psq[PIECE_NB][SQUARE_NB + 1];
@@ -33,7 +33,7 @@ extern Score psq[PIECE_NB][SQUARE_NB + 1];
 // Fill psqt array from a set of internally linked parameters
 extern void init(const Variant*);
 
-} // namespace PSQT
+} // namespace Stockfish::PSQT
 
 
 #endif // PSQT_H_INCLUDED
