@@ -89,6 +89,8 @@
 #  define pext(b, m) 0
 #endif
 
+namespace Stockfish {
+
 #ifdef USE_POPCNT
 constexpr bool HasPopCnt = true;
 #else
@@ -181,6 +183,10 @@ struct Bitboard {
         return Bitboard(~b64[0], ~b64[1]);
     }
 
+    constexpr Bitboard operator - () const {
+        return Bitboard(-b64[0] - (b64[1] > 0), -b64[1]);
+    }
+
     constexpr Bitboard operator | (const Bitboard x) const {
         return Bitboard(b64[0] | x.b64[0], b64[1] | x.b64[1]);
     }
@@ -370,7 +376,6 @@ enum Value : int {
   WazirValueMg             = 400,   WazirValueEg             = 350,
   CommonerValueMg          = 700,   CommonerValueEg          = 900,
   CentaurValueMg           = 1800,  CentaurValueEg           = 1900,
-  Tempo = 28,
 
   MidgameLimit  = 15258, EndgameLimit  = 3915
 };
@@ -403,6 +408,8 @@ static_assert(2 * SQUARE_BITS + MOVE_TYPE_BITS + 2 * PIECE_TYPE_BITS <= 32, "Mov
 
 enum Piece {
   NO_PIECE,
+  W_PAWN = PAWN,                 W_KNIGHT, W_BISHOP, W_ROOK, W_QUEEN, W_KING = KING,
+  B_PAWN = PAWN + PIECE_TYPE_NB, B_KNIGHT, B_BISHOP, B_ROOK, B_QUEEN, B_KING = KING + PIECE_TYPE_NB,
   PIECE_NB = 2 * PIECE_TYPE_NB
 };
 
@@ -529,18 +536,13 @@ struct DirtyPiece {
   // to from SQ_NONE to the capture square.
   Piece piece[12];
   Piece handPiece[12];
+  int handCount[12];
 
   // From and to squares, which may be SQ_NONE
   Square from[12];
   Square to[12];
 };
 
-enum NnueFeatures {
-  NNUE_CHESS,
-  NNUE_SHOGI,
-  NNUE_VARIANT,
-};
-
 /// Score enum stores a middlegame and an endgame value in a single integer (enum).
 /// The least significant 16 bits are used to store the middlegame value and the
 /// upper 16 bits are used to store the endgame value. We have to take care to
@@ -809,6 +811,8 @@ constexpr Key make_key(uint64_t seed) {
   return seed * 6364136223846793005ULL + 1442695040888963407ULL;
 }
 
+} // namespace Stockfish
+
 #endif // #ifndef TYPES_H_INCLUDED
 
 #include "tune.h" // Global visibility to tuning setup
