@@ -1,9 +1,12 @@
-use std::fmt;
-use std::io;
-use std::fs::{File, OpenOptions};
-use std::path::{Path, PathBuf};
-use serde::Serialize;
+use std::{
+    fmt,
+    fs::{File, OpenOptions},
+    io,
+    path::{Path, PathBuf},
+};
+
 use bitflags::bitflags;
+use serde::Serialize;
 use tempfile::TempDir;
 use xz2::read::XzDecoder;
 
@@ -16,7 +19,7 @@ struct Asset {
 
 impl Asset {
     #[cfg(unix)]
-    fn open_executable_file(&self, path: &Path) -> io::Result<File> {
+    fn open_executable_file(path: &Path) -> io::Result<File> {
         use std::os::unix::fs::OpenOptionsExt as _;
         OpenOptions::new()
             .create(true)
@@ -26,23 +29,20 @@ impl Asset {
     }
 
     #[cfg(not(unix))]
-    fn open_executable_file(&self, path: &Path) -> io::Result<File> {
-        self.open_file(path)
+    fn open_executable_file(path: &Path) -> io::Result<File> {
+        Asset::open_file(path)
     }
 
-    fn open_file(&self, path: &Path) -> io::Result<File> {
-        OpenOptions::new()
-            .create(true)
-            .write(true)
-            .open(path)
+    fn open_file(path: &Path) -> io::Result<File> {
+        OpenOptions::new().create(true).write(true).open(path)
     }
 
     fn create(&self, base: &Path) -> io::Result<PathBuf> {
         let path = base.join(self.name);
         let mut file = if self.executable {
-            self.open_executable_file(&path)
+            Asset::open_executable_file(&path)
         } else {
-            self.open_file(&path)
+            Asset::open_file(&path)
         }?;
 
         let mut decoder = XzDecoder::new(self.data);
@@ -87,27 +87,30 @@ impl Cpu {
         cpu.set(Cpu::POPCNT, is_x86_feature_detected!("popcnt"));
         cpu.set(Cpu::SSE41, is_x86_feature_detected!("sse4.1"));
         cpu.set(Cpu::AVX2, is_x86_feature_detected!("avx2"));
-        cpu.set(Cpu::FAST_BMI2, is_x86_feature_detected!("bmi2") && {
-            let cpuid = raw_cpuid::CpuId::new();
-            match cpuid.get_vendor_info() {
-                // Intel was implementing BMI2 in hardware from the beginning.
-                Some(vendor) if vendor.as_str() == "GenuineIntel" => true,
-                // Due to patents, AMD was using slow software emulation
-                // for PEXT for a long time. The Zen 3 family (0x19) is the
-                // first to implement it in hardware.
-                Some(vendor) if vendor.as_str() == "AuthenticAMD" => {
-                    cpuid.get_feature_info().map_or(false, |f| {
-                        let family = if f.family_id() == 15 {
-                            f.extended_family_id() + f.family_id()
-                        } else {
-                            f.family_id()
-                        };
-                        family >= 0x19
-                    })
-                },
-                _ => false,
-            }
-        });
+        cpu.set(
+            Cpu::FAST_BMI2,
+            is_x86_feature_detected!("bmi2") && {
+                let cpuid = raw_cpuid::CpuId::new();
+                match cpuid.get_vendor_info() {
+                    // Intel was implementing BMI2 in hardware from the beginning.
+                    Some(vendor) if vendor.as_str() == "GenuineIntel" => true,
+                    // Due to patents, AMD was using slow software emulation
+                    // for PEXT for a long time. The Zen 3 family (0x19) is the
+                    // first to implement it in hardware.
+                    Some(vendor) if vendor.as_str() == "AuthenticAMD" => {
+                        cpuid.get_feature_info().map_or(false, |f| {
+                            let family = if f.family_id() == 15 {
+                                f.extended_family_id() + f.family_id()
+                            } else {
+                                f.family_id()
+                            };
+                            family >= 0x19
+                        })
+                    }
+                    _ => false,
+                }
+            },
+        );
         cpu
     }
 
@@ -140,7 +143,10 @@ const STOCKFISH: &[Asset] = &[
     },
     Asset {
         name: "stockfish-x86-64-sse41-popcnt",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/stockfish-x86-64-sse41-popcnt.xz")),
+        data: include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/stockfish-x86-64-sse41-popcnt.xz"
+        )),
         needs: Cpu::SF_SSE41_POPCNT,
         executable: true,
     },
@@ -168,7 +174,10 @@ const STOCKFISH_MV: &[Asset] = &[
     },
     Asset {
         name: "fairy-stockfish-x86-64-sse41-popcnt",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/fairy-stockfish-x86-64-sse41-popcnt.xz")),
+        data: include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/fairy-stockfish-x86-64-sse41-popcnt.xz"
+        )),
         needs: Cpu::SF_SSE41_POPCNT,
         executable: true,
     },
@@ -196,7 +205,10 @@ const STOCKFISH: &[Asset] = &[
     },
     Asset {
         name: "stockfish-x86-64-sse41-popcnt.exe",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/stockfish-x86-64-sse41-popcnt.exe.xz")),
+        data: include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/stockfish-x86-64-sse41-popcnt.exe.xz"
+        )),
         needs: Cpu::SF_SSE41_POPCNT,
         executable: true,
     },
@@ -212,19 +224,28 @@ const STOCKFISH: &[Asset] = &[
 const STOCKFISH_MV: &[Asset] = &[
     Asset {
         name: "fairy-stockfish-x86-64-bmi2.exe",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/fairy-stockfish-x86-64-bmi2.exe.xz")),
+        data: include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/fairy-stockfish-x86-64-bmi2.exe.xz"
+        )),
         needs: Cpu::SF_BMI2,
         executable: true,
     },
     Asset {
         name: "fairy-stockfish-x86-64-avx2.exe",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/fairy-stockfish-x86-64-avx2.exe.xz")),
+        data: include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/fairy-stockfish-x86-64-avx2.exe.xz"
+        )),
         needs: Cpu::SF_AVX2,
         executable: true,
     },
     Asset {
         name: "fairy-stockfish-x86-64-sse41-popcnt.exe",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/fairy-stockfish-x86-64-sse41-popcnt.exe.xz")),
+        data: include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/fairy-stockfish-x86-64-sse41-popcnt.exe.xz"
+        )),
         needs: Cpu::SF_SSE41_POPCNT,
         executable: true,
     },
@@ -237,45 +258,39 @@ const STOCKFISH_MV: &[Asset] = &[
 ];
 
 #[cfg(all(target_os = "linux", target_arch = "aarch64"))]
-const STOCKFISH: &[Asset] = &[
-    Asset {
-        name: "stockfish-armv8",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/stockfish-armv8.xz")),
-        needs: Cpu::empty(),
-        executable: true,
-    },
-];
+const STOCKFISH: &[Asset] = &[Asset {
+    name: "stockfish-armv8",
+    data: include_bytes!(concat!(env!("OUT_DIR"), "/stockfish-armv8.xz")),
+    needs: Cpu::empty(),
+    executable: true,
+}];
 
 #[cfg(all(target_os = "linux", target_arch = "aarch64"))]
-const STOCKFISH_MV: &[Asset] = &[
-    Asset {
-        name: "fairy-stockfish-armv8",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/fairy-stockfish-armv8.xz")),
-        needs: Cpu::empty(),
-        executable: true,
-    },
-];
-
+const STOCKFISH_MV: &[Asset] = &[Asset {
+    name: "fairy-stockfish-armv8",
+    data: include_bytes!(concat!(env!("OUT_DIR"), "/fairy-stockfish-armv8.xz")),
+    needs: Cpu::empty(),
+    executable: true,
+}];
 
 #[cfg(all(target_os = "macos", target_arch = "aarch64"))]
-const STOCKFISH: &[Asset] = &[
-    Asset {
-        name: "stockfish-apple-silicon",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/stockfish-apple-silicon.xz")),
-        needs: Cpu::empty(),
-        executable: true,
-    },
-];
+const STOCKFISH: &[Asset] = &[Asset {
+    name: "stockfish-apple-silicon",
+    data: include_bytes!(concat!(env!("OUT_DIR"), "/stockfish-apple-silicon.xz")),
+    needs: Cpu::empty(),
+    executable: true,
+}];
 
 #[cfg(all(target_os = "macos", target_arch = "aarch64"))]
-const STOCKFISH_MV: &[Asset] = &[
-    Asset {
-        name: "fairy-stockfish-apple-silicon",
-        data: include_bytes!(concat!(env!("OUT_DIR"), "/fairy-stockfish-apple-silicon.xz")),
-        needs: Cpu::empty(),
-        executable: true,
-    },
-];
+const STOCKFISH_MV: &[Asset] = &[Asset {
+    name: "fairy-stockfish-apple-silicon",
+    data: include_bytes!(concat!(
+        env!("OUT_DIR"),
+        "/fairy-stockfish-apple-silicon.xz"
+    )),
+    needs: Cpu::empty(),
+    executable: true,
+}];
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
 pub enum EngineFlavor {
@@ -334,24 +349,35 @@ impl EvalFlavor {
 
 #[derive(Debug)]
 pub struct Assets {
-    dir: TempDir,
     pub sf_name: &'static str,
     pub nnue: String,
     pub stockfish: ByEngineFlavor<PathBuf>,
+    _dir: TempDir, // Will be deleted when dropped
 }
 
 impl Assets {
     pub fn prepare(cpu: Cpu) -> io::Result<Assets> {
         let dir = tempfile::Builder::new().prefix("fishnet-").tempdir()?;
-        let sf = STOCKFISH.iter().find(|a| cpu.contains(a.needs)).expect("compatible stockfish");
+        let sf = STOCKFISH
+            .iter()
+            .find(|a| cpu.contains(a.needs))
+            .expect("compatible stockfish");
         Ok(Assets {
-            nnue: NNUE.create(dir.path())?.to_str().expect("nnue path printable").to_owned(),
+            nnue: NNUE
+                .create(dir.path())?
+                .to_str()
+                .expect("nnue path printable")
+                .to_owned(),
             sf_name: sf.name,
             stockfish: ByEngineFlavor {
                 official: sf.create(dir.path())?,
-                multi_variant: STOCKFISH_MV.iter().find(|a| cpu.contains(a.needs)).expect("compatible stockfish").create(dir.path())?,
+                multi_variant: STOCKFISH_MV
+                    .iter()
+                    .find(|a| cpu.contains(a.needs))
+                    .expect("compatible stockfish")
+                    .create(dir.path())?,
             },
-            dir,
+            _dir: dir,
         })
     }
 }
