@@ -1,24 +1,43 @@
-use std::cmp::{max, min};
-use std::collections::{HashMap, VecDeque};
-use std::collections::hash_map::Entry;
-use std::convert::TryInto;
-use std::sync::Arc;
-use std::time::{Duration, Instant};
-use shakmaty::{CastlingMode, Position as _, PositionError};
-use shakmaty::uci::{IllegalUciError, Uci};
-use shakmaty::variant::VariantPosition;
-use tokio::sync::{mpsc, Mutex, Notify, oneshot};
-use tokio::time;
-use url::Url;
-use crate::api::{Acquired, AcquireQuery, AcquireResponseBody, AnalysisPart, ApiStub, BatchId, LichessVariant, Work};
-use crate::assets::{EngineFlavor, EvalFlavor};
-use crate::configure::{BacklogOpt, Endpoint};
-use crate::ipc::{Position, PositionFailed, PositionId, PositionResponse, Pull};
-use crate::logger::{Logger, ProgressAt, QueueStatusBar};
-use crate::util::{NevermindExt as _, RandomizedBackoff};
-use crate::stats::{StatsRecorder, Stats, NpsRecorder};
+use std::{
+    cmp::{max, min},
+    collections::{hash_map::Entry, HashMap, VecDeque},
+    convert::TryInto,
+    sync::Arc,
+    time::{Duration, Instant},
+};
 
-pub fn channel(opt: BacklogOpt, cores: usize, api: ApiStub, max_backoff: Duration, logger: Logger) -> (QueueStub, QueueActor) {
+use shakmaty::{
+    fen::Fen,
+    uci::{IllegalUciError, Uci},
+    variant::VariantPosition,
+    CastlingMode, Position as _, PositionError,
+};
+use tokio::{
+    sync::{mpsc, oneshot, Mutex, Notify},
+    time,
+};
+use url::Url;
+
+use crate::{
+    api::{
+        AcquireQuery, AcquireResponseBody, Acquired, AnalysisPart, ApiStub, BatchId,
+        LichessVariant, Work,
+    },
+    assets::{EngineFlavor, EvalFlavor},
+    configure::{BacklogOpt, Endpoint},
+    ipc::{Position, PositionFailed, PositionId, PositionResponse, Pull},
+    logger::{Logger, ProgressAt, QueueStatusBar},
+    stats::{NpsRecorder, Stats, StatsRecorder},
+    util::{NevermindExt as _, RandomizedBackoff},
+};
+
+pub fn channel(
+    opt: BacklogOpt,
+    cores: usize,
+    api: ApiStub,
+    max_backoff: Duration,
+    logger: Logger,
+) -> (QueueStub, QueueActor) {
     let (tx, rx) = mpsc::unbounded_channel();
     let interrupt = Arc::new(Notify::new());
     let state = Arc::new(Mutex::new(QueueState::new(cores, logger.clone())));
@@ -57,9 +76,8 @@ impl QueueStub {
         }
         if let Err(callback) = state.try_pull(callback) {
             if let Some(ref mut tx) = self.tx {
-                tx.send(QueueMessage::Pull {
-                    callback,
-                }).nevermind("queue dropped");
+                tx.send(QueueMessage::Pull { callback })
+                    .nevermind("queue dropped");
             }
         }
     }
@@ -91,7 +109,10 @@ impl QueueStub {
 
     pub async fn stats(&self) -> (Stats, NpsRecorder) {
         let state = self.state.lock().await;
-        (state.stats_recorder.stats.clone(), state.stats_recorder.nnue_nps.clone())
+        (
+            state.stats_recorder.stats.clone(),
+            state.stats_recorder.nnue_nps.clone(),
+        )
     }
 }
 
@@ -127,20 +148,26 @@ impl QueueState {
 
     fn add_incoming_batch(&mut self, batch: IncomingBatch) {
         match self.pending.entry(batch.work.id()) {
-            Entry::Occupied(entry) => self.logger.error(&format!("Dropping duplicate incoming batch {}", entry.key())),
+            Entry::Occupied(entry) => self.logger.error(&format!(
+                "Dropping duplicate incoming batch {}",
+                entry.key()
+            )),
             Entry::Vacant(entry) => {
                 let progress_at = ProgressAt::from(&batch);
 
                 // Reversal only for cosmetics when displaying progress.
                 let mut positions = Vec::with_capacity(batch.positions.len());
                 for pos in batch.positions.into_iter().rev() {
-                    positions.insert(0, match pos {
-                        Skip::Present(pos) => {
-                            self.incoming.push_back(pos);
-                            None
-                        }
-                        Skip::Skip => Some(Skip::Skip),
-                    });
+                    positions.insert(
+                        0,
+                        match pos {
+                            Skip::Present(pos) => {
+                                self.incoming.push_back(pos);
+                                None
+                            }
+                            Skip::Skip => Some(Skip::Skip),
+                        },
+                    );
                 }
 
                 entry.insert(PendingBatch {
@@ -157,7 +184,11 @@ impl QueueState {
         }
     }
 
-    fn handle_position_response(&mut self, queue: QueueStub, res: Result<PositionResponse, PositionFailed>) {
+    fn handle_position_response(
+        &mut self,
+        queue: QueueStub,
+        res: Result<PositionResponse, PositionFailed>,
+    ) {
         match res {
             Ok(res) => {
                 let progress_at = ProgressAt::from(&res);
@@ -180,7 +211,10 @@ impl QueueState {
         }
     }
 
-    fn try_pull(&mut self, callback: oneshot::Sender<Position>) -> Result<(), oneshot::Sender<Position>> {
+    fn try_pull(
+        &mut self,
+        callback: oneshot::Sender<Position>,
+    ) -> Result<(), oneshot::Sender<Position>> {
         if let Some(position) = self.incoming.pop_front() {
             if let Err(err) = callback.send(position) {
                 self.incoming.push_front(err);
@@ -202,20 +236,42 @@ impl QueueState {
                     }
                     extra.push(match completed.nps() {
                         Some(nps) => {
-                            let nnue_nps = if completed.flavor.eval_flavor() == EvalFlavor::Nnue { Some(nps) } else { None };
-                            self.stats_recorder.record_batch(completed.total_positions(), completed.total_nodes(), nnue_nps);
+                            let nnue_nps = if completed.flavor.eval_flavor() == EvalFlavor::Nnue {
+                                Some(nps)
+                            } else {
+                                None
+                            };
+                            self.stats_recorder.record_batch(
+                                completed.total_positions(),
+                                completed.total_nodes(),
+                                nnue_nps,
+                            );
                             format!("{} knps", nps / 1000)
                         }
                         None => "? nps".to_owned(),
                     });
                     let log = match completed.url {
-                        Some(ref url) => format!("{} {} finished ({})", self.status_bar(), url, extra.join(", ")),
-                        None => format!("{} {} finished ({})", self.status_bar(), batch, extra.join(", ")),
+                        Some(ref url) => format!(
+                            "{} {} finished ({})",
+                            self.status_bar(),
+                            url,
+                            extra.join(", ")
+                        ),
+                        None => format!(
+                            "{} {} finished ({})",
+                            self.status_bar(),
+                            batch,
+                            extra.join(", ")
+                        ),
                     };
                     match completed.work {
                         Work::Analysis { id, .. } => {
                             self.logger.info(&log);
-                            queue.api.submit_analysis(id, completed.flavor.eval_flavor(), completed.into_analysis());
+                            queue.api.submit_analysis(
+                                id,
+                                completed.flavor.eval_flavor(),
+                                completed.into_analysis(),
+                            );
                         }
                         Work::Move { .. } => {
                             self.logger.debug(&log);
@@ -228,8 +284,14 @@ impl QueueState {
                     if !pending.work.matrix_wanted() {
                         // Send partially analysis as progress report.
                         let progress_report = pending.progress_report();
-                        if progress_report.iter().filter(|p| p.is_some()).count() % (self.cores * 2) == 0 {
-                            queue.api.submit_analysis(pending.work.id(), pending.flavor.eval_flavor(), progress_report);
+                        if progress_report.iter().filter(|p| p.is_some()).count() % (self.cores * 2)
+                            == 0
+                        {
+                            queue.api.submit_analysis(
+                                pending.work.id(),
+                                pending.flavor.eval_flavor(),
+                                progress_report,
+                            );
                         }
                     }
 
@@ -242,9 +304,7 @@ impl QueueState {
 
 #[derive(Debug)]
 enum QueueMessage {
-    Pull {
-        callback: oneshot::Sender<Position>,
-    },
+    Pull { callback: oneshot::Sender<Position> },
     MoveSubmitted,
 }
 
@@ -270,20 +330,28 @@ impl QueueActor {
             let state = self.state.lock().await;
             state.stats_recorder.min_user_backlog()
         };
-        let user_backlog = max(min_user_backlog, self.opt.user.map(Duration::from).unwrap_or_default());
+        let user_backlog = max(
+            min_user_backlog,
+            self.opt.user.map(Duration::from).unwrap_or_default(),
+        );
         let system_backlog = self.opt.system.map(Duration::from).unwrap_or_default();
 
         if user_backlog >= sec || system_backlog >= sec {
             if let Some(status) = self.api.status().await {
-                let user_wait = user_backlog.checked_sub(status.user.oldest).unwrap_or_default();
-                let system_wait = system_backlog.checked_sub(status.system.oldest).unwrap_or_default();
+                let user_wait = user_backlog
+                    .checked_sub(status.user.oldest)
+                    .unwrap_or_default();
+                let system_wait = system_backlog
+                    .checked_sub(status.system.oldest)
+                    .unwrap_or_default();
                 self.logger.debug(&format!("User wait: {:?} due to {:?} for oldest {:?}, system wait: {:?} due to {:?} for oldest {:?}",
                        user_wait, user_backlog, status.user.oldest,
                        system_wait, system_backlog, status.system.oldest));
                 let slow = user_wait >= system_wait + sec;
                 (min(user_wait, system_wait), AcquireQuery { slow })
             } else {
-                self.logger.debug("Queue status not available. Will not delay acquire.");
+                self.logger
+                    .debug("Queue status not available. Will not delay acquire.");
                 let slow = user_backlog >= system_backlog + sec;
                 (Duration::default(), AcquireQuery { slow })
             }
@@ -305,11 +373,17 @@ impl QueueActor {
                 state.add_incoming_batch(incoming);
             }
             Err(IncomingError::AllSkipped(completed)) => {
-                self.logger.warn(&format!("Completed empty batch {}.", context));
-                self.api.submit_analysis(completed.work.id(), completed.flavor.eval_flavor(), completed.into_analysis());
+                self.logger
+                    .warn(&format!("Completed empty batch {}.", context));
+                self.api.submit_analysis(
+                    completed.work.id(),
+                    completed.flavor.eval_flavor(),
+                    completed.into_analysis(),
+                );
             }
             Err(err) => {
-                self.logger.warn(&format!("Ignoring invalid batch {}: {:?}", context, err));
+                self.logger
+                    .warn(&format!("Ignoring invalid batch {}: {:?}", context, err));
             }
         }
     }
@@ -329,7 +403,11 @@ impl QueueActor {
             };
 
             if let Some(completed) = next {
-                if let Some(Acquired::Accepted(body)) = self.api.submit_move_and_acquire(completed.work.id(), completed.into_best_move()).await {
+                if let Some(Acquired::Accepted(body)) = self
+                    .api
+                    .submit_move_and_acquire(completed.work.id(), completed.into_best_move())
+                    .await
+                {
                     self.handle_acquired_response_body(body).await;
                 }
             } else {
@@ -341,68 +419,66 @@ impl QueueActor {
     async fn run_inner(mut self) {
         while let Some(msg) = self.rx.recv().await {
             match msg {
-                QueueMessage::Pull { mut callback } => {
-                    loop {
-                        self.handle_move_submissions().await;
+                QueueMessage::Pull { mut callback } => loop {
+                    self.handle_move_submissions().await;
 
-                        {
-                            let mut state = self.state.lock().await;
-                            callback = match state.try_pull(callback) {
-                                Ok(()) => break,
-                                Err(not_done) => not_done,
-                            };
-
-                            if state.shutdown_soon {
-                                break;
-                            }
-                        }
-
-                        let (wait, query) = tokio::select! {
-                            _ = callback.closed() => break,
-                            res = self.backlog_wait_time() => res,
+                    {
+                        let mut state = self.state.lock().await;
+                        callback = match state.try_pull(callback) {
+                            Ok(()) => break,
+                            Err(not_done) => not_done,
                         };
 
-                        if wait >= Duration::from_secs(1) {
-                            if wait >= Duration::from_secs(40) {
-                                self.logger.info(&format!("Going idle for {:?}.", wait));
-                            } else {
-                                self.logger.debug(&format!("Going idle for {:?}.", wait));
-                            }
-
-                            tokio::select! {
-                                _ = callback.closed() => break,
-                                _ = self.interrupt.notified() => continue,
-                                _ = time::sleep(wait) => continue,
-                            }
-                        }
-
-                        match self.api.acquire(query).await {
-                            Some(Acquired::Accepted(body)) => {
-                                self.backoff.reset();
-                                self.handle_acquired_response_body(body).await;
-                            }
-                            Some(Acquired::NoContent) => {
-                                let backoff = self.backoff.next();
-                                self.logger.debug(&format!("No job received. Backing off {:?}.", backoff));
-                                tokio::select! {
-                                    _ = callback.closed() => break,
-                                    _ = self.interrupt.notified() => (),
-                                    _ = time::sleep(backoff) => (),
-                                }
-                            }
-                            Some(Acquired::Rejected) => {
-                                self.logger.error("Client update or reconfiguration might be required. Stopping queue.");
-                                let mut state = self.state.lock().await;
-                                state.shutdown_soon = true;
-                            },
-                            None => (),
+                        if state.shutdown_soon {
+                            break;
                         }
                     }
-                }
+
+                    let (wait, query) = tokio::select! {
+                        _ = callback.closed() => break,
+                        res = self.backlog_wait_time() => res,
+                    };
+
+                    if wait >= Duration::from_secs(1) {
+                        if wait >= Duration::from_secs(40) {
+                            self.logger.info(&format!("Going idle for {:?}.", wait));
+                        } else {
+                            self.logger.debug(&format!("Going idle for {:?}.", wait));
+                        }
+
+                        tokio::select! {
+                            _ = callback.closed() => break,
+                            _ = self.interrupt.notified() => continue,
+                            _ = time::sleep(wait) => continue,
+                        }
+                    }
+
+                    match self.api.acquire(query).await {
+                        Some(Acquired::Accepted(body)) => {
+                            self.backoff.reset();
+                            self.handle_acquired_response_body(body).await;
+                        }
+                        Some(Acquired::NoContent) => {
+                            let backoff = self.backoff.next();
+                            self.logger
+                                .debug(&format!("No job received. Backing off {:?}.", backoff));
+                            tokio::select! {
+                                _ = callback.closed() => break,
+                                _ = self.interrupt.notified() => (),
+                                _ = time::sleep(backoff) => (),
+                            }
+                        }
+                        Some(Acquired::Rejected) => {
+                            self.logger.error("Client update or reconfiguration might be required. Stopping queue.");
+                            let mut state = self.state.lock().await;
+                            state.shutdown_soon = true;
+                        }
+                        None => (),
+                    }
+                },
                 QueueMessage::MoveSubmitted => self.handle_move_submissions().await,
             }
         }
-
     }
 }
 
@@ -434,23 +510,41 @@ pub struct IncomingBatch {
 }
 
 impl IncomingBatch {
-    fn from_acquired(endpoint: &Endpoint, body: AcquireResponseBody) -> Result<IncomingBatch, IncomingError> {
+    fn from_acquired(
+        endpoint: &Endpoint,
+        body: AcquireResponseBody,
+    ) -> Result<IncomingBatch, IncomingError> {
         let url = body.batch_url(endpoint);
 
-        let maybe_pos = VariantPosition::from_setup(body.variant.into(), &body.position, CastlingMode::Chess960);
+        let maybe_root_pos = VariantPosition::from_setup(
+            body.variant.into(),
+            &body.position,
+            CastlingMode::Chess960,
+        );
 
-        let (flavor, mut pos) = match maybe_pos {
-            Ok(pos @ VariantPosition::Chess(_)) if body.work.is_analysis() => (EngineFlavor::Official, pos),
+        let (flavor, root_pos) = match maybe_root_pos {
+            Ok(pos @ VariantPosition::Chess(_)) if body.work.is_analysis() => {
+                (EngineFlavor::Official, pos)
+            }
             Ok(pos) => (EngineFlavor::MultiVariant, pos),
-            Err(pos) => (EngineFlavor::MultiVariant, pos.ignore_impossible_material()?),
+            Err(pos) => (
+                EngineFlavor::MultiVariant,
+                pos.ignore_impossible_material()?,
+            ),
         };
 
-        let mut body_moves = Vec::new();
-        for uci in body.moves {
-            let m = uci.to_move(&pos)?;
-            body_moves.push(m.to_uci(CastlingMode::Chess960));
-            pos.play_unchecked(&m);
-        }
+        let root_fen = Fen::from_setup(&root_pos);
+
+        let body_moves = {
+            let mut moves = Vec::with_capacity(body.moves.len());
+            let mut pos = root_pos;
+            for uci in body.moves {
+                let m = uci.to_move(&pos)?;
+                moves.push(m.to_uci(CastlingMode::Chess960));
+                pos.play_unchecked(&m);
+            }
+            moves
+        };
 
         Ok(IncomingBatch {
             work: body.work.clone(),
@@ -465,7 +559,7 @@ impl IncomingBatch {
                         flavor,
                         position_id: PositionId(0),
                         variant: body.variant,
-                        fen: body.position,
+                        root_fen,
                         moves: body_moves,
                     })]
                 }
@@ -480,7 +574,7 @@ impl IncomingBatch {
                         flavor,
                         position_id: PositionId(0),
                         variant: body.variant,
-                        fen: body.position.clone(),
+                        root_fen: root_fen.clone(),
                         moves: moves.clone(),
                     })];
 
@@ -495,7 +589,7 @@ impl IncomingBatch {
                             flavor,
                             position_id: PositionId(1 + i),
                             variant: body.variant,
-                            fen: body.position.clone(),
+                            root_fen: root_fen.clone(),
                             moves: moves.clone(),
                         }));
                     }
@@ -523,7 +617,7 @@ impl IncomingBatch {
 
                     positions
                 }
-            }
+            },
         })
     }
 }
@@ -584,12 +678,16 @@ impl PendingBatch {
     }
 
     fn progress_report(&self) -> Vec<Option<AnalysisPart>> {
-        self.positions.iter().enumerate().map(|(i, p)| match p {
-            // Quirk: Lila distinguishes progress reports from complete
-            // analysis by looking at the first part.
-            Some(Skip::Present(pos)) if i > 0 => Some(pos.to_best()),
-            _ => None,
-        }).collect()
+        self.positions
+            .iter()
+            .enumerate()
+            .map(|(i, p)| match p {
+                // Quirk: Lila distinguishes progress reports from complete
+                // analysis by looking at the first part.
+                Some(Skip::Present(pos)) if i > 0 => Some(pos.to_best()),
+                _ => None,
+            })
+            .collect()
     }
 
     fn pending(&self) -> usize {
@@ -610,13 +708,16 @@ pub struct CompletedBatch {
 
 impl CompletedBatch {
     fn into_analysis(self) -> Vec<Option<AnalysisPart>> {
-        self.positions.into_iter().map(|p| {
-            Some(match p {
-                Skip::Skip => AnalysisPart::Skipped { skipped: true },
-                Skip::Present(pos) if pos.work.matrix_wanted() => pos.into_matrix(),
-                Skip::Present(pos) => pos.to_best(),
+        self.positions
+            .into_iter()
+            .map(|p| {
+                Some(match p {
+                    Skip::Skip => AnalysisPart::Skipped { skipped: true },
+                    Skip::Present(pos) if pos.work.matrix_wanted() => pos.into_matrix(),
+                    Skip::Present(pos) => pos.to_best(),
+                })
             })
-        }).collect()
+            .collect()
     }
 
     fn into_best_move(self) -> Option<Uci> {
@@ -627,22 +728,29 @@ impl CompletedBatch {
     }
 
     fn total_positions(&self) -> u64 {
-        self.positions.iter().map(|p| match p {
-            Skip::Skip => 0,
-            Skip::Present(_) => 1,
-        }).sum()
+        self.positions
+            .iter()
+            .map(|p| match p {
+                Skip::Skip => 0,
+                Skip::Present(_) => 1,
+            })
+            .sum()
     }
 
     fn total_nodes(&self) -> u64 {
-        self.positions.iter().map(|p| match p {
-            Skip::Skip => 0,
-            Skip::Present(pos) => pos.nodes,
-        }).sum()
+        self.positions
+            .iter()
+            .map(|p| match p {
+                Skip::Skip => 0,
+                Skip::Present(pos) => pos.nodes,
+            })
+            .sum()
     }
 
     fn nps(&self) -> Option<u32> {
-        self.completed_at.checked_duration_since(self.started_at).and_then(|time| {
-            (u128::from(self.total_nodes()) * 1000).checked_div(time.as_millis())
-        }).and_then(|nps| nps.try_into().ok())
+        self.completed_at
+            .checked_duration_since(self.started_at)
+            .and_then(|time| (u128::from(self.total_nodes()) * 1000).checked_div(time.as_millis()))
+            .and_then(|nps| nps.try_into().ok())
     }
 }
