@@ -1,5 +1,8 @@
-use std::cmp::{max, min};
-use std::time::Duration;
+use std::{
+    cmp::{max, min},
+    time::Duration,
+};
+
 use rand::Rng;
 
 #[derive(Debug)]
